% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TE.R
\name{tangent-elliptical}
\alias{tangent-elliptical}
\alias{d_TE}
\alias{TE}
\alias{r_TE}
\title{Tangent elliptical distribution}
\usage{
d_TE(x, theta, g_scaled, d_V, Lambda, log = FALSE)

r_TE(n, theta, r_V, Lambda)
}
\arguments{
\item{x}{locations in \eqn{S^{p-1}} to evaluate the density. Either a
matrix of size \code{c(nx, p)} or a vector of length \code{p}. Normalized
internally if required (with a \code{warning} message).}

\item{theta}{a unit norm vector of size \code{p} giving the axis of
rotational symmetry.}

\item{g_scaled}{the \emph{scaled} angular density \eqn{c_g g}. In the
form \cr\code{g_scaled <- function(t, log = TRUE) {...}}. See examples.}

\item{d_V}{the density \eqn{f_V}. In the form
\code{d_V <- function(v, log = TRUE) {...}}. See examples.}

\item{Lambda}{the shape matrix \eqn{\boldsymbol{\Lambda}}{\Lambda} of the
ACG used in the multivariate signs. A symmetric and positive definite
matrix of size \code{c(p - 1, p - 1)}.}

\item{log}{flag to indicate if the logarithm of the density (or the
normalizing constant) is to be computed.}

\item{n}{sample size, a positive integer.}

\item{r_V}{a function for simulating \eqn{V}. Its first argument must be
the sample size. See examples.}
}
\value{
Depending on the function:
\itemize{
\item \code{d_TE}: a vector of length \code{nx} or \code{1} with the
evaluated density at \code{x}.
\item \code{r_TE}: a matrix of size \code{c(n, p)} with the random sample.
}
}
\description{
Density and simulation of the Tangent Elliptical (TE)
distribution on
\eqn{S^{p-1}:=\{\mathbf{x}\in R^p:||\mathbf{x}||=1\}}{
S^{p-1} := \{x \in R^p : ||x|| = 1\}}, \eqn{p\ge 2}. The distribution arises
by considering the
\link[=tang-norm-decomp]{tangent-normal decomposition} with
multivariate \link[=cosines-signs]{signs} distributed as an
\link[=ACG]{Angular Central Gaussian} distribution.
}
\details{
The functions are wrappers for \code{\link{d_tang_norm}} and
\code{\link{r_tang_norm}} with \code{d_U = \link{d_ACG}} and
\code{r_U = \link{r_ACG}}.
}
\examples{
## Simulation and density evaluation for p = 2

# Parameters
p <- 2
n <- 1e3
theta <- c(rep(0, p - 1), 1)
Lambda <- matrix(0.5, nrow = p - 1, ncol = p - 1)
diag(Lambda) <- 1
kappa_V <- 2

# Required functions
r_V <- function(n) r_g_vMF(n = n, p = p, kappa = kappa_V)
g_scaled <- function(t, log) {
  g_vMF(t, p = p - 1, kappa = kappa_V, scaled = TRUE, log = log)
}

# Sample and color according to density
x <- r_TE(n = n, theta = theta, r_V = r_V, Lambda = Lambda)
col <- viridisLite::viridis(n)
r <- runif(n, 0.95, 1.05) # Radius perturbation to improve visualization
dens <- d_TE(x = x, theta = theta, g_scaled = g_scaled, Lambda = Lambda)
plot(r * x, pch = 16, col = col[rank(dens)])

## Simulation and density evaluation for p = 3

# Parameters
p <- 3
n <- 5e3
theta <- c(rep(0, p - 1), 1)
Lambda <- matrix(0.5, nrow = p - 1, ncol = p - 1)
diag(Lambda) <- 1
kappa_V <- 2

# Sample and color according to density
x <- r_TE(n = n, theta = theta, r_V = r_V, Lambda = Lambda)
col <- viridisLite::viridis(n)
dens <- d_TE(x = x, theta = theta, g_scaled = g_scaled, Lambda = Lambda)
rgl::plot3d(x, col = col[rank(dens)], size = 5)

## A non-vMF angular function: g(t) = 1 - t^2. It is sssociated to the
## Beta(1/2, (p + 1)/2) distribution.

# Scaled angular function
g_scaled <- function(t, log) {
  log_c_g <- lgamma(0.5 * p) + log(0.5 * p / (p - 1)) - 0.5 * p * log(pi)
  log_g <- log_c_g + log(1 - t^2)
  switch(log + 1, exp(log_g), log_g)
}

# Simulation
r_V <- function(n) {
  sample(x = c(-1, 1), size = n, replace = TRUE) *
    sqrt(rbeta(n = n, shape1 = 0.5, shape2 = 0.5 * (p + 1)))
}

# Sample and color according to density
kappa_V <- 1
Lambda <- matrix(0.75, nrow = p - 1, ncol = p - 1)
diag(Lambda) <- 1
x <- r_TE(n = n, theta = theta, r_V = r_V, Lambda = Lambda)
col <- viridisLite::viridis(n)
dens <- d_TE(x = x, theta = theta, g_scaled = g_scaled, Lambda = Lambda)
rgl::plot3d(x, col = col[rank(dens)], size = 5)
}
\references{
García-Portugués, E., Paindaveine, D., Verdebout, T. (2019) On optimal tests
for rotational symmetry against new classes of hyperspherical distributions.
\emph{arXiv:1706.05030}. \url{https://arxiv.org/abs/1706.05030}
}
\seealso{
\code{\link{tang-norm-decomp}},
\code{\link{tangent-vMF}}, \code{\link{ACG}}.
}
\author{
Eduardo García-Portugués, Davy Paindaveine, and Thomas Verdebout.
}
